from snappy.snappy import Snappy

class TestSnappy:
    
    # Uncomment to perform snapshot test
    # def test_snappy_snap_roots(self):
        
    #     # Arrange
    #     instances = ["172.31.255.50", "172.31.255.20"]
    #     tags_specifications = [{
    #         "Key": "CreatorName",
    #         "Value": "mervin.hemaraju@checkout.com"
    #     }]
        
    #     # Act
    #     try:
    #         snappy = Snappy(instances)
            
    #         snapshots = snappy.snap_roots(tags_specifications)
            
    #         print(f"test_snappy_snap_roots: {snapshots}")
            
    #     except Exception as e:
    #         result = str(e)
            
    #         print(f"test_snappy_snap_roots: {result}")
            
    #     # Assert
    #     assert False
    
    def test_snappy_initialization_AbnormalData(self):
        
        # Arrange
        instances = ["172.31.255.50", "10.0.0.0"]
        expected_result = "The following instances could not be retrieved: ['10.0.0.0']"
        
        # Act
        try:
            snappy = Snappy(instances)
            
            result = "Failed"
        except Exception as e:
            result = str(e)
            
        # Assert
        assert result == expected_result
    
    def test_snappy_initialization_NormalData(self):
        
        # Arrange
        instances = ["172.31.255.50", "172.31.255.20"]
        expected_result = "Passed"
        
        # Act
        try:
            snappy = Snappy(instances)
            
            result = "Passed"
        except Exception as e:
            result = str(e)
            
        # Assert
        assert result == expected_result


import pytest
from snappy.utils.helper import has_errors, retrieve_failed_instances


###########################
####### Test Models #######
###########################
class MyInstanceT:
    
    def __init__(self, private_ip) -> None:
        self.private_ip = private_ip
        
class TestHelper:
    
    testdata = [
        (["X","X","X"], ["X","X"], True),
        (["X"], [], True),
        ([], ["X"], True),
        (["X","X","X"], ["X","X","X"], False),
        (["X"], ["X"], False),
        ([], [], False),
    ]
    @pytest.mark.parametrize("previous_instances,retrieved_instances,expected_result", testdata)
    def test_has_errors(self, previous_instances, retrieved_instances, expected_result):
        
        # Arrange
        
        # Act
        result = has_errors(previous_instances, retrieved_instances)
        
        # Assert
        assert expected_result == result
    
    
    testdata = [
        (
            ["10.10.10.1", "10.10.10.2","10.10.10.3", "10.10.10.4"], 
            [MyInstanceT("10.10.10.1"), MyInstanceT("10.10.10.3")], 
            ["10.10.10.2", "10.10.10.4"]
        ),
        (
            ["10.10.10.1", "10.10.10.2","10.10.10.3","10.10.10.4"], 
            [MyInstanceT("10.10.10.1"), MyInstanceT("10.10.10.2"), MyInstanceT("10.10.10.3")], 
            ["10.10.10.4"]
        ),
        (
            ["10.10.10.1",], 
            [], 
            ["10.10.10.1"]
        ),
        (
            ["10.10.10.1", "10.10.10.2","10.10.10.3", "10.10.10.4"], 
            [MyInstanceT("10.10.10.1"), MyInstanceT("10.10.10.2"), MyInstanceT("10.10.10.3"), MyInstanceT("10.10.10.4")], 
            []
        ),
        (
            ["10.10.10.1", "10.10.10.2","10.10.10.3", "10.10.10.4"], 
            [], 
            ["10.10.10.1", "10.10.10.2","10.10.10.3", "10.10.10.4"]
        ),
    ]
    @pytest.mark.parametrize("previous_instances,retrieved_instances,expected_result", testdata)
    def test_retrieve_failed_instancess(self, previous_instances, retrieved_instances, expected_result):
        
        # Arrange
        
        # Act
        result = retrieve_failed_instances(previous_instances, retrieved_instances)
        
        # Assert
        assert expected_result == result




def has_errors(previous_instances, retrieved_instances):
    
    # Check if some instances were not retrieved
    return len(previous_instances) != len(retrieved_instances)

def retrieve_failed_instances(previous_instances, retrieved_instances):
    
    # Retrieve the private IPs to manipulate
    retrieved_instances_ip = [instance.private_ip for instance in retrieved_instances]
    
    # Return all ip addresses not located in the retrieved instances
    return [ip for ip in previous_instances if ip not in retrieved_instances_ip]


import boto3
import  snappy.utils.constants as Consts
from snappy.instance import Instance
from snappy.utils import helper as Helper

class Snappy():

    def __init__(self, instances):

        # Create an empty list of instances
        self.instances = []

        # Create the boto3 EC2 client
        client = boto3.client('ec2')

        # Describe all instances
        response = client.describe_instances(
            Filters=Consts.filter_boto3_instance_retrieval(instances)
        )

        # Filter and append Instances
        for r in response['Reservations']:

            for i in r['Instances']:

                self.instances.append(Instance(i))
                
        # Verify if all instances were retrieved successfully
        if Helper.has_errors(instances, self.instances):
            
            # Retrieve the failing instances
            failed_instances = Helper.retrieve_failed_instances(instances, self.instances)
            
            # Raise the exception
            raise Exception(Consts.EXCEPTION_MESSAGE_INSTANCES_RETRIEVAL_FAILED.format(str(failed_instances)))
        
    def snap_roots(self, tags_specifications=None):
        
        # Make root snapshots for each instances and return the list of output
        return [instance.snap_root(tags_specifications) for instance in self.instances]